/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.basicrpc ;

import javax.servlet.http.*;

import org.azuki.bean.basicrpc.server.imp.*;
import org.azuki.bean.basicrpc.shared.ConstBasicRpc;
import org.azuki.client.basicrpc.Sender;
import org.azuki.common.bean.rpc.AbstractRpcBean;
import org.azuki.common.consts.Const;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;
import org.jdom.Element;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class BasicRpcBean extends AbstractRpcBean
{
	@DependencyInjectionInfo(desc="Check if user/password exists", param={String.class, Object.class}, mandatory=false)
	public IMethodProxy<Boolean> check ;
	
	private boolean secu = true ;
	final public boolean isSecured()	{ return secu ; }

	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	public void postCreate(Element lmen)
	throws Exception
	{
		// security
		Element secu = JDOMUtil.getElement(lmen, ConstBasicRpc.SECU) ;
		
		String enabStr = secu.getAttributeValue(ConstBasicRpc.SECU_ENAB) ;
		if(enabStr==null)
		{	enabStr = Const.YES ;
			Log.warning("\tNo authentication configuration found, using default (YES)") ; }
		
		this.secu = Util.isOK(enabStr) ;
		
		Log.info("\tAuthentication enabled: "+(this.secu?Const.YES:Const.NO)) ;
		
		// Method injection is done before initialization
		if(this.secu && check==null)
			throw new Exception("Server error : missing authentication method configuration") ;
	}
	
	final static private String S_URI_SEND       = "/"+Sender.SEND ;
	final static private String S_URI_BATCH      = "/"+Sender.BATCH ;
	final static private String S_URI_CONNECT    = "/"+Sender.CONNECT ;
	final static private String S_URI_DISCONNECT = "/"+Sender.DISCONNECT ;
	final static private String S_URI_LOG        = "/"+Sender.LOG ;
	
	@MethodInfo(desc="The contract implementation of the request dispatcher aspect (RequestDispatcher)")
	final public void service(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		try
		{
			String uri = req.getRequestURI() ;
			
	    	if(uri.endsWith(S_URI_CONNECT)) 	// connect method
	    	{	PService.doConnect(this, req, res) ;
	    		return ; }
	    	
	    	if(uri.endsWith(S_URI_DISCONNECT)) // disconnect method
	    	{	PService.doDisconnect(this, req, res) ;
	    		return ; }
	    	
	    	if(uri.endsWith(S_URI_LOG)) 		// log method
	    	{	PService.doLog(this, req, res) ;
	    		return ; }
	    	
	    	IContext cx = ThreadContext.get() ;

			// Authentication check
			if(isSecured())
			{
				try
				{
					HttpSession sess = req.getSession(false) ;
					if(sess==null) throw new Exception("No valid session.") ;
					
					Object user = sess.getAttribute(IContext.ATTR_USER) ;
					if(user==null) throw new Exception("No user defined.") ;

					cx.bind(IContext.ATTR_USER, user, new ProcessLifecycle()) ;
				}
				catch(Exception ex)
				{
					Log.info(ex.getMessage()) ;
					Response.write(req, res, ex) ;
					return ;
				}
			}
			
			cx.bind(IContext.OBJECT_HTTPREQUEST, req, null) ; // If you need information from the original servlet request
			
			// Method invocation
			if(uri.endsWith(S_URI_SEND))
	    	{	PService.doCall(this, req, res) ;
	    		return ; }
			
			if(uri.endsWith(S_URI_BATCH))
			{	PService.doBatch(this, req, res) ;
				return ; }
			
			throw new Exception("Not implemented") ;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			Response.write(req, res, ex) ;
			throw ex ;
		}
	}
}