/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.basicrpc.server.imp ;

import java.util.Arrays;
import java.util.logging.Level;

import javax.servlet.http.*;

import org.azuki.bean.basicrpc.BasicRpcBean;
import org.azuki.common.bean.rpc.Service;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.context.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class PService
{
	final static public void doConnect(BasicRpcBean bean, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		if(bean.check==null)
			throw new Exception("Server error : missing authentication method configuration") ;
		
		Object ores = null ;
		
		try
		{
			// Read of java serialize parameters
			Object[] param = (Object[])Http.readObject(req) ;

			String user = (String)param[0] ; 
			Object args = param[1] ;
			
			ores = bean.check.call(user, args) ;
			
			if(!Boolean.TRUE.equals(ores))
				throw new Exception("Not a valid user: "+user) ;
			
			HttpSession sess = req.getSession(true) ; // Session creation
			sess.setAttribute(IContext.ATTR_USER, user) ;
		}
		catch(Throwable t) // Error are serialized to the client
		{
			ores = t ;
		}
		
		Response.write(req, res, ores) ;
	}
	
	final static public void doDisconnect(BasicRpcBean bean, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Object ores = null ;
		
		try
		{
			HttpSession sess = req.getSession(false) ;
			if(sess!=null) sess.removeAttribute(IContext.ATTR_USER) ;
		}
		catch(Throwable t)
		{
			ores = t ;
		}
		
		Response.write(req, res, ores) ;
	}
	
	final static public void doLog(BasicRpcBean bean, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		try
		{
			HttpSession sess = req.getSession(false) ;
			String user = sess==null ? "unknown" : (String)sess.getAttribute(IContext.ATTR_USER) ;
		
			// Read of java serialize parameters
			Object[] param = (Object[])Http.readObject(req) ;
 
			String plvl = (String)param[0] ;
			String pmsg = (String)param[1] ;
			String pthr = (String)param[2] ;
			
			Level levl = Level.FINEST ;
			try
			{ levl = Level.parse(plvl) ; }
			catch(Exception ex)
			{ Log.severe(ex.getMessage()) ; }
			
			String nfo = new StringBuilder()
			.append("Session(").append(sess==null?"none":sess.getId()).append(") ")
			.append("User(").append(user).append(")").toString() ;
			
			StringBuilder bdr = new StringBuilder() ;
			bdr.append("--> START LOG ").append(nfo).append("\n") ;
			bdr.append(pmsg).append("\n") ;
			if(!StringUtil.isEmpty(pthr)) bdr.append(pthr).append("\n") ;
			bdr.append("<-- END LOG ").append(nfo) ;
			
			Log.getLogger().log(levl, bdr.toString()) ;
		}
		catch(Exception ex)
		{
			Log.severe("Cannot parse client log: "+ex.getMessage()) ;
		}
		finally
		{
			Response.write(req, res, null) ; // don't throw exception, otherwise loop !!
		}
	}
	
	final static public void doCall(BasicRpcBean bean, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Object ores = null ;

		try
		{
			// Read of java serialize parameters
			Object[] param = (Object[])Http.readObject(req) ;

			String service = (String)  param[0] ; 
			Object[] args  = (Object[])param[1] ; 
			
	    	Log.info("Invoking : "+service) ;
	    	
	    	Service serv = bean.getService(service) ;
			if(serv==null) throw new Exception("Service not found: " + service) ;
	    	 	 
			ores = serv.call(args) ;
		}
		catch(Throwable t) // Error are serialized to the client
		{
			Log.severe(t) ;
			ores = t ;
		}
		
		Response.write(req, res, ores) ;    	
	}
	
	final static public void doBatch(BasicRpcBean bean, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Object ores = null ;
		
		try
		{
			Log.info("--> START BATCH INVOKER") ;	
			
			Object[] lines = (Object[])Http.readObject(req) ;
			
			for(Object line : lines)
			{
				Object[] sarr = (Object[])line ;
				
				String   snam = (String)sarr[0] ;
				Object[] args = (Object[])sarr[1] ;
				
				Service serv = bean.getService(snam) ;
				if(serv==null) throw new Exception("Service not found: "+snam) ;
				
				Log.info("call "+serv) ;
				Log.finest("with arguments: "+(args==null ? null : Arrays.asList(args))) ;
				
				serv.call(args) ;
			}
			
			Log.info("<-- END BATCH INVOKER") ;
		}
		catch(Throwable t)
		{
			Log.severe(t) ;
			ores = t ;
		}
		
		Response.write(req, res, ores) ;
	}	
}