/*
 *   Created the 1 August 2007
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.cxf ;

import java.lang.reflect.Method;
import java.util.*;

import javax.jws.WebService;
import javax.servlet.http.*;
import javax.xml.ws.Endpoint;

import org.azuki.bean.cxf.server.CXFWSAdvice;
import org.azuki.bean.cxf.server.console.CXFWSConsole;
import org.azuki.bean.cxf.server.imp.PSource;
import org.azuki.bean.cxf.shared.ConstCXF;
import org.azuki.common.consts.Tag;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;
import org.jdom.Element;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
@WebService
final public class CXFWSBean implements IAspect<CXFWSAdvice>
{
	final static public String NAME = "WebService" ;
	
	final static private String webservicePackage = "org.azuki.webservice" ;
	
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	@DependencyInjectionInfo(desc="The method that convert source code into a class",
							 param={String.class, Object.class, List.class})
	public IMethodProxy<Class> compiler ;

	final private Map<String,Service> srvs = new HashMap<String,Service>() ;  	// Object[]{String beanName, String beanLabel, Method method}
	final private List<Service> errs = new ArrayList<Service>() ;				// Object[]{String service, String beanName, String beanLabel, Method method}
	
	final public Service getService(String name)
	{
		return srvs.get(name) ;
	}
	
	final public Collection<Service> getServices()
	{
		return srvs.values() ;
	}
	
	final public Collection<Service> getServicesInErrors()
	{
		return Collections.unmodifiableCollection(errs) ;
	}
	
	//
	// IAspect
	//
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public CXFWSAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		getMetadata().getContract().checkBindable(tmeta.getBeanClass()) ;
		
		for(Method m : ipc.getSelectedMethods())
		{
			String snam = m.getName() ;
			
			Element mpar = ipc.getElement(m) ;
			
			Element para = mpar.getChild(Tag.PARA) ;
			if(para!=null)
			{
				Element lmen = para.getChild(ConstCXF.SERV) ;
				
				String temp = lmen.getAttributeValue(ConstCXF.SERV_NAME) ;
				if(!StringUtil.isEmpty(temp)) snam = temp ;
			}
			
			Service serv = new Service(snam, tmeta.getName(), tmeta.getLabel(), m) ;
			
			if(srvs.containsKey(snam))
			{	Log.severe("Service("+snam+") already registered") ;
				errs.add(serv) ;
				continue ; }
			
			srvs.put(snam, serv) ;
			
			Log.info("Registering: "+serv) ;
		}
		
		return new CXFWSAdvice() ;
	}
	
	// Must be called once, in CXFServlet startup
	final public void create()
	throws Exception 
	{
		IMetadata myMeta = getMetadata() ;
		
		String labl = myMeta.getLabel() ;
		
		String code = PSource.mkCode(labl, srvs.values(), webservicePackage) ;
		Log.finest(code) ;
		
		if(compiler==null)
			throw new Exception("Missing compiler method injection") ;
		
		Class c = compiler.call(webservicePackage+'.'+labl, code, null) ; //getFrameworkClasspath()
	
		Log.info("Create dynamic class ("+c.getName()+"): "+c.getPackage()) ;
		
		Object inst = c.newInstance() ;
		
		Log.info("Bind webservice ("+labl+") --> "+inst) ;
		Endpoint.publish("/"+labl, inst) ;
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{		
		CXFWSConsole.p(this, srv, req, res) ;
	}
	
	//
	// Member class
	//
	final static public class Service
	{
		final public String serv ;
		final public String name ;
		final public String labl ;
		final public Method meth ;
		
		public Service(String serv, String name, String labl, Method meth)
		{
			this.serv = serv ;
			this.name = name ;
			this.labl = labl ;
			this.meth = meth ;
		}
		
		@Override
		final public String toString()
		{
			return "Service("+serv+") --> ("+Util.oneString(name, labl)+"."+meth.getName()+")" ;
		}
	}
	
//	//
//	// STATIC
//	//
//	private static List<String> fwcp = null ;
//	
//	@SuppressWarnings("unchecked")
//	final static private List<String> getFrameworkClasspath()
//	{
//		if(fwcp!=null) return fwcp ;
//		
//		String clas = Framework.class.getName().replace('.', '/')+".class" ;
//
//		java.net.URL furl = Framework.class.getClassLoader().getResource(clas) ;
//		
//		String ff = furl.getPath() ;
//		
//		int indx = ff.indexOf('!') ;
//		if(indx!=-1) return Collections.singletonList(ff.substring(0, indx)) ;
//		
//		indx = ff.indexOf(clas) ;
//		fwcp = indx==-1 ? Collections.EMPTY_LIST : Collections.singletonList(ff.substring(0, indx)) ;
//		
//		return fwcp ;
//	}
}