/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.injection.server.imp ;

import java.lang.reflect.*;
import java.util.*;

import org.azuki.common.reflect.ReflectUtil;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.command.ICommand;
import org.azuki.framework.injection.*;

public class MethodInvocation<T> implements ICommand<T>, InvocationTarget<T>
{
	final private IMetadata meta ;
	
	final private String   meth ;
	
	private Class[]  cs   ; // can be null
	private Object[] args ; // can be null
	
	public MethodInvocation(IMetadata meta, String meth, Class[] cs, Object... args)
	{
		this.meta = meta ;
		this.meth = meth ;
		this.cs   = cs ;
		this.args = args ;
	}
	
	private Object bean ;
	private IController[] ctrls ;
	private int stage = 0 ;	
    private HashMap<Object,Object> map = null ; // Lazy 	

	final public T process()
	throws Exception
	{
		bean  = meta.getBean() ;
		ctrls = meta.getControllers() ;
		
		return proceed() ;
	}
	
	//////////////////////////////////////////////////
	// Chain of Control
	
	final public Object getBean()   { return bean; }
	final public String getMethodName() { return meth; }

	// Entry point for a method call, this method execute fist
	// a chain of controller ...
	
	@SuppressWarnings("unchecked")
	final public T proceed() 
	throws Exception
	{
		int length = ctrls==null?0:ctrls.length ;
		
		while(stage<length)
			return (T)ctrls[stage++].intercept(this) ;
		
		if(cs!=null)
			return (T)callExact(bean, meth, cs, args) ;
			
		return (T)callBest(bean, meth, args) ;
	}
	
	final public void setData(Object key, Object val)
	{
		if(map==null) map=new HashMap<Object,Object>() ; 
		map.put(key, val); 
	}
	
	final public Object getData(Object key)
	{
		if(map==null) map=new HashMap<Object,Object>() ; 
		return map.get(key);
	} 

	final public Map getDatas()
	{
		if(map==null) return new HashMap();
		return (Map)map.clone(); 
	}

	// TODO : get/set parameter type ?
	
	final public Object[] getParameters()
	{
		if(args==null) return null ;
		return args.clone() ;
	}

	final public void setParameters(Object[] args)
	{
		cs = null ; // Switch to Best effort
		this.args = args ;
	}	
	

	// Invoke simple (seulement si on est sure que les types des parametres match exactement)
	final static private <T>Object callExact(Object o, String method, Class[] cs, Object... args)
	throws Exception
	{	
		Method m = o.getClass().getMethod(method, cs) ; 

		try
		{
			return m.invoke(o, args) ;		
		}
		catch(InvocationTargetException ite)
		{
//			//Log.severe(Log.getStack(ite.getTargetException())) ;
//			throw new Exception(ite.getTargetException()) ; // Reel cause
			throw new Exception(getCause(ite)) ;
		}
	}
	
	// Invoke avec type argument indefini
	final static private <T>Object callBest(Object bean, String method, Object... vargs)
	throws Exception
	{
		Class c = bean.getClass() ;
		
		Method m = ReflectUtil.bestMatch(c, method, vargs) ;
		
		if(m==null)
			throw new Exception("Undefined method ["+method+"]") ;

		try
		{
			vargs = convert(m, vargs) ;
			
			return m.invoke(bean, vargs) ;
		}
		catch(InvocationTargetException ite)
		{
//			throw new Exception(ite) ; 
////			throw new Exception(ite.getTargetException()) ; // Reel cause
			
			throw new Exception(getCause(ite)) ;
		}
	}
	
	// Convertion dans les type attendus par la method pour certain primitif
	final static private Object[] convert(Method m, Object... vargs)
	throws Exception
	{
		if(vargs==null) return null ;
		
		Class[] cs = m.getParameterTypes() ;
		if(vargs.length!=cs.length)
		{
			if(!m.isVarArgs()) return vargs ; // Conversion impossible
			
			if(vargs.length<cs.length-1) return vargs ; // Conversion impossible
			
			Object[] clon = Arrays.copyOf(vargs, cs.length) ;
				
			if(vargs.length>=cs.length)
			{
				int pos = cs.length-1 ;
				int len = vargs.length-cs.length+1 ;
				
				Object[] varg = new Object[len] ;
				System.arraycopy(vargs, pos, varg, 0, len) ;
				
				clon[pos] = varg ;
			}
				
			vargs = clon ;
		}
		
		// 
		int i = 0 ;
		for(Class<?> c : cs)
		{
			int cur = i++ ;
			Object arg = vargs[cur] ;
			if(arg==null) continue ; // no problemo
			
			Class<?> argc = arg.getClass() ;
			if(c==argc) continue ; // itou
			if(c.isAssignableFrom(argc)) continue ; // itou
					
			// Try to convert parameters
			vargs[cur] = convert(c, arg) ;
		}
				
		return vargs ;
	}
	
	final static private Object convert(Class<?> c, Object arg)
	throws Exception
	{
		if(arg==null) return null ;
		
		try
		{
			if(c==String.class) return arg.toString() ;
			
			if(c==Integer.class || c==int.class)
			{
				String str = arg.toString() ; 
				if(str.endsWith(".0")) str = str.substring(0, str.indexOf('.')) ;
				return new Integer(str) ;
			}
			
			if(c==Long.class || c==long.class)
			{
				String str = arg.toString() ; 
				if(str.endsWith(".0")) str = str.substring(0, str.indexOf('.')) ;
				return new Long(str) ;
			}
			
			if(c==Float.class || c==float.class)	return new Float(arg.toString()) ;
			if(c==Double.class || c==double.class) 	return new Double(arg.toString()) ;
			if(c==Byte.class || c==byte.class)		return new Byte(arg.toString()) ;
			if(c==Character.class || c==char.class)	return new Character(arg.toString().charAt(0)) ;
			
			if(c==Boolean.class || c==boolean.class) return new Boolean(arg.toString()) ;
			
			if(c==Date.class)
			{
				String str = arg.toString() ; 
				if(str.endsWith(".0")) str = str.substring(0, str.indexOf('.')) ;
				return new Date(Long.parseLong(str)) ;
			}
			
			if(c.isArray() && arg.getClass().isArray()) return convertArray(c, arg) ;
		}
		catch(Throwable t)
		{
			throw new Exception("Cannot convert: "+arg.getClass().getName()+" --> "+c.getName(), t) ;
		}
		
		return arg ;
	}
	
	final static public Object convertArray(Class<?> c, Object arg)
	throws Exception
	{
		if(!c.isArray()) return arg ;
		if(!arg.getClass().isArray()) return arg ;
		
		int len = Array.getLength(arg) ;
			
		Class<?> ctyp = c.getComponentType() ;
		
		Object arr = Array.newInstance(ctyp, len) ;
		for(int i=0 ; i<len ; i++)
		{
			Object val = Array.get(arg, i) ;		
			Array.set(arr, i, convert(ctyp, val)) ;
		}
			
		return arr ;
	}
	
	final static public Throwable getCause(InvocationTargetException ite)
	{
		while(true)
		{
			Throwable t = ite.getTargetException() ;
			if(t instanceof InvocationTargetException)
			{	ite = (InvocationTargetException)t ;
				continue ; }
			
			return t==null ? ite : t ; // be sure to not return null !
		}
	}
}