/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.jms;

import java.io.Serializable;
import java.util.Map;

import javax.jms.*;
import javax.naming.*;

import org.azuki.bean.jms.shared.ConstJMS;
import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

/**
 * a generic bean that send object to a broker.
 * could be weaved with injection 
 * could be executed throw EBP .
 * use two parameters:
 *    - JNDI_CONTEXT_FACTORY : JNDI lookup String for the broker
 *    - DESTINATION : a JNDI lookup string for a queue or a topic 
 * @author mjucker
 *
 */
public class JMSSenderBean
{
	private String JNDI_CONTEXT_FACTORY;
	private String DESTINATION;
	
	private boolean isValid = false;
	
	private ConnectionFactory qf;
	private Connection qc;
	private Session ses;
	private Context JNDIContext;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		try
		{
		JNDIContext = new InitialContext();
	
		Map<String,String> map = EditorUtil.formToMap(para);
		if (map==null)
		{
			Log.severe("Bean parameters are missing !");
			throw new Exception("Bean parameters are missing !");
		}
		
		JNDI_CONTEXT_FACTORY = map.get(ConstJMS.KEY_JNDI_CONTEXT_FACTORY);
		DESTINATION=map.get(ConstJMS.KEY_DESTINATION);
		
		if (JNDI_CONTEXT_FACTORY==null || JNDI_CONTEXT_FACTORY.isEmpty())
		{
			Log.severe("JNDI_CONTEXT_FACTORY name is not set : can't get any ContextFactory");
			throw new Exception("JNDI_CONTEXT_FACTORY name is not set : can't get any ContextFactory");
		}
		Log.info(JNDI_CONTEXT_FACTORY);
		if (DESTINATION==null)
		{
			Log.severe("DESTINATION name is not set : can't read any destination");
			throw new Exception("DESTINATION name is not set : can't read any destination");
		}
		
		isValid = true;

	
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw e;
		}
	}
	
	@Export
	public void sendObject(Object msg)
	throws Exception
	{
		try
		{
			Log.info("receive event to send an objet to a broker...");
			if (!isValid) 
			{
				Log.severe("parameters are invalid");
				closeAll();
				return ;
			}
			
			if (!Serializable.class.isAssignableFrom(msg.getClass()))
			{
				Log.severe("Object to send is not serializable : can't send Message to JMS Broker");
				closeAll();
				return;
			}
			
			qf = (ConnectionFactory) JNDIContext.lookup("java:/comp/env/"+JNDI_CONTEXT_FACTORY);
			if (qf==null )
			{
				Log.severe("JMS ContextFactory is null : can't get a JMS Connection");
				return;
			}
			
			initConnection();
			
			//QUEUE or TOPIC
			
			Destination dest = (Destination) JNDIContext.lookup("java:/comp/env/"+DESTINATION);
			if (dest==null)
			{
				Log.severe("JMS Destination is null : JNDI lookup failed");
				closeAll();
				return ;
			}
			Log.info("send to Broker...");
			ObjectMessage tm = ses.createObjectMessage((Serializable)msg);
			tm.setJMSTimestamp(System.currentTimeMillis());
			
			ses.createProducer(dest).send(tm);
			ses.commit();
			closeAll();
			Log.info("[OK]");
		}
		catch (Exception e)
		{
			e.printStackTrace();
			Log.severe("Error while trying to send a Message to JMS Broker" + Log.getStack(e));
			throw e;
		}
	}
	
	final private void initConnection() throws Exception
	{
		qc = qf.createConnection();

		if (qc==null)
		{
			Log.severe("JMS Connection is null : createConnection failed");
			return ;
		}
		
		ses = qc.createSession(true,Session.AUTO_ACKNOWLEDGE);
		
		if (ses==null)
		{
			Log.severe("JMS Session is null : createSession failed");
			closeAll();
			throw new Exception("JMS Session is null : createSession failed");
		}
		qc.start();
	}
	
	final private void closeAll() throws JMSException
	{
		if (ses!=null)
			ses.close();
		if (qc!=null)
		{
			qc.stop();
			qc.close();
		}
	}
}
