/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.jms;

import java.util.Map;
import java.util.logging.*;

import javax.jms.*;
import javax.naming.*;
import javax.servlet.http.*;

import org.azuki.bean.jms.server.console.JMSTestToolConsole;
import org.azuki.bean.jms.shared.ConstJMS;
import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

public class JMSTestToolBean
{
	private String JNDI_CONTEXT_FACTORY;
	private Logger logger;
	
	int i = 0;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		Map<String,String> param = EditorUtil.formToMap(para);
		if (param==null)
			throw new Exception("no parameter are defined !!! bean can't work");
		
		JNDI_CONTEXT_FACTORY=param.get(ConstJMS.KEY_JNDI_CONTEXT_FACTORY);		
	}
	

	final public ConnectionFactory testConnectionFactory()
	throws Exception
	{
		if (JNDI_CONTEXT_FACTORY==null || JNDI_CONTEXT_FACTORY.isEmpty())
		{
			logger.log(Level.SEVERE,"JNDI_CONTEXT_FACTORY name is not set : can't get any ContextFactory");
			throw new Exception("JNDI_CONTEXT_FACTORY name is not set : can't get any ContextFactory");
		}
		Context JNDIContext = new InitialContext();
		ConnectionFactory qf = (ConnectionFactory) JNDIContext.lookup("java:/comp/env/"+JNDI_CONTEXT_FACTORY);
	
		if (qf==null )
		{
			logger.log(Level.SEVERE,"JMS ContextFactory is null : can't get a JMS Connection");
			throw new Exception(this.getClass().getName()+"ERROR:JMS ContextFactory is null : can't get a JMS Connection");
		}
		return qf;
		
	}
	
	final public Connection testConnection()
	throws Exception
	{
		ConnectionFactory qf = testConnectionFactory();
		
		Connection qc = qf.createConnection();
		if (qc==null)
		{
			logger.log(Level.SEVERE,"JMS Connection is null : createConnection failed");
			throw new Exception(this.getClass().getName()+"ERROR:JMS Connection is null : createConnection failed");
		}
		return qc;
	}
	
	final public Session testSession()
	throws Exception
	{
		Connection qc = testConnection();
		Session ses = qc.createSession(true,Session.AUTO_ACKNOWLEDGE);
		if (ses==null)
		{
			logger.log(Level.SEVERE,"JMS Session is null : createSession failed");
			throw new Exception(this.getClass().getName()+"ERROR:JMS Session is null : createSession failed");
		}
		return ses;
	}
	
	final private Session getSession(Connection qc)
	throws Exception
	{
		Session ses = qc.createSession(true,Session.AUTO_ACKNOWLEDGE);
		if (ses==null)
		{
			logger.log(Level.SEVERE,"JMS Session is null : createSession failed");
			throw new Exception(this.getClass().getName()+"ERROR:JMS Session is null : createSession failed");
		}
		return ses;
	}
	
	/**
	 * int : status :
	 *     0 : successfull
	 *     2 : send failed
	 *     4 : read failed
	 *     16: other
	 *      
	 * @return
	 * @throws Exception
	 */
	final public int testQueueConnection() 
	{
		try
		{
			Connection qc = testConnection();
			Session ses = getSession(qc);
			qc.start();
			
			// create a temporary queue
			TemporaryQueue q = ses.createTemporaryQueue();
			
			try
			{
				// test a post in queue
				MessageProducer producer = ses.createProducer(q);
				TextMessage txt=ses.createTextMessage("JMS queue test");
				producer.send(q,txt);
				ses.commit();
				producer.close();
			}
			catch(JMSException jmse)
			{
				Log.severe("test send message in JMS queue failed");
				jmse.printStackTrace();
				if (ses!=null)
					ses.close();
				if (qc!=null)
				{
					qc.stop();
					qc.close();
				}
				return 2;
			}
			ses.close();
			try
			{
				ses = getSession(qc);
				MessageConsumer consumer = ses.createConsumer(q);
				TextMessage txt = (TextMessage) consumer.receive(500);
				if (txt==null || !txt.getText().equals("JMS queue test"))
				{
					Log.severe("test read message in JMS queue failed");
					if (consumer!=null)
						consumer.close();
					if (ses!=null)
						ses.close();
					if (qc!=null)
					{
						qc.stop();
						qc.close();
					}
					return 4;
				}
				consumer.close();
			}
			catch(JMSException jmse)
			{
				Log.severe("test read message in JMS queue failed");
				Log.severe(jmse);
				if (ses!=null)
					ses.close();
				if (qc!=null)
				{
					qc.stop();
					qc.close();
				}
				return 4;
			}
			
			
			// delete temporary queue
			q.delete();
			ses.close();
			qc.close();
		}
		catch(Exception e)
		{
			Log.severe(e);
			return 16;
		}
		return 0;
	}
	
	/**
	 * int : status :
	 *     0 : successfull
	 *     2 : send failed
	 *     4 : read failed
	 *     16: other
	 *      
	 * @return
	 * @throws Exception
	 */
	final public int testTopicConnection() 
	{
		try
		{
			Connection qc = testConnection();
			
			IMetadata meta = Bean.getMetadata(this);
			String CLIENT_ID = meta.getID().toString()+(++i);
			qc.setClientID(CLIENT_ID);

			Session ses = getSession(qc);
			qc.start();
			
			TemporaryTopic q = ses.createTemporaryTopic();
			
			MessageProducer mp = ses.createProducer(q);
			MessageConsumer mc = ses.createConsumer(q);
			
			// create a temporary topic
			Log.info("temporaty topic name="+q.getTopicName());
			try
			{
				// test a post in topic
				TextMessage txt=ses.createTextMessage("JMS topic test");
				Log.info("send message into a topic");
				mp.send(q,txt);
				ses.commit();
				Log.info("send message successfull");
				mp.close();
				
				// try 3 time
				while(i<3)
				{
					txt = (TextMessage) mc.receive(500);
					ses.commit();
					if (txt!=null) break;
					i++;
				}
				//
				mc.close();
				//
				if (txt==null || !txt.getText().equals("JMS topic test"))
				{
					Log.info("TextMessage is null !!!");
					Log.severe("test read message in JMS topic failed");
					if (ses!=null)
						ses.close();
					if (qc!=null)
					{
						qc.stop();
						qc.close();
					}
					return 4;
				}
				Log.info("message retrieved from topic");
				
			}
			catch(JMSException jmse)
			{
				Log.severe("test send message in JMS topic failed");
				Log.severe(jmse);
				
				if (mc!=null)
					mc.close();
				if (mp!=null)
					mp.close();
				if (ses!=null)
					ses.close();
				if (qc!=null)
				{
					qc.stop();
					qc.close();
				}
				return 2;
			}
			
			// delete temporary topic
			q.delete();
			ses.close();
			qc.stop();
			qc.close();
		}
		catch(Exception e)
		{
			Log.severe("error occure while during Topic test");
			Log.severe(e);
			return 16;
		}
		return 0;
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet servlet, HttpServletRequest req , HttpServletResponse res)
	throws Exception
	{
		JMSTestToolConsole.service(this,servlet,req,res);
	}
}
