/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.jmx.server.imp ;

import java.lang.annotation.Annotation;
import java.lang.reflect.*;
import java.util.*;

import javax.management.*;

import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.context.Log;


public class MBeanInfoFactory
{
	final static public MBeanInfo mkInfo(IPointcut ipc)
	{
		MBeanInfo mBeanInfo = null ;
		
		try
		{
			List<MBeanNotificationInfo> notfInfos = new ArrayList<MBeanNotificationInfo>() ;
			
			Log.info("\t(*) Build MBeanInfo ");
			
			// register operation
			MBeanOperationInfo[] opeinfo = computeMBeanOperationInfo(notfInfos, ipc.getSelectedMethods()) ;
			
			// register Attributes + attribute notification definition
			MBeanAttributeInfo[] attrinfo = computeAttributeInfo(notfInfos, ipc.getSelectedFields()) ;
			
			// retrieve defined notifications
			MBeanNotificationInfo[] notfinfo = notfInfos.toArray(new MBeanNotificationInfo[notfInfos.size()]) ;
			
			// create mbean infos
			IMetadata tmeta = ipc.getTargetMetadata() ;
			String cnam = tmeta.getBean().getClass().getCanonicalName() ;
			String desc = tmeta.getDescription() ;
			mBeanInfo = new MBeanInfo(cnam, desc, attrinfo, null, opeinfo, notfinfo) ;
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return mBeanInfo;
	}

	final static private MBeanOperationInfo[] computeMBeanOperationInfo(List<MBeanNotificationInfo> notfInfos, Method[] methods)
	throws Exception
	{
		if(methods==null) return null ;
		
		System.out.println("\t\t (*) ComputeOperationInfo") ;
		
		List<MBeanOperationInfo> operNfos = new ArrayList<MBeanOperationInfo>() ;
		
		for(Method method : methods)
		{
			String methName = method.getName() ;
			
			String methDesc = methName ; // Default value if no annotation
			MethodInfo mi = method.getAnnotation(MethodInfo.class) ;
			if(mi!=null)
				if(!"".equals(mi.desc())) methDesc = mi.desc() ;
			
			
//			MBeanParameterInfo[] paramNfos = JMXUtil.getMethodParameters(method) ;
			MBeanParameterInfo[] paramNfos = new MBeanParameterInfo[method.getParameterTypes().length] ;
			
			int i = 0 ; // param counter
			Class[] types = method.getParameterTypes() ;
			
			Annotation[][] pass =  method.getParameterAnnotations() ;
			for(Annotation[] pas : pass) // pa : table of parameter annotations for one parameter 
			{
				String tnam  = types[i].getCanonicalName() ;
					
				// Default parameter info, if no @ArgInfo annotation
				MBeanParameterInfo mbpi = new MBeanParameterInfo("p"+i, tnam, null) ;

				for(Annotation pa : pas)
				{
					// Recherche de l'annotation ArgInfo
					if(ArgInfo.class!=pa.annotationType()) continue ;
					ArgInfo p = (ArgInfo)pa ;
					
					mbpi = new MBeanParameterInfo(p.name(), tnam, p.desc()) ;
					
					break ;
				}
				
				paramNfos[i++] = mbpi ;
			}
			
			String methType = method.getReturnType().getCanonicalName() ; // uniformiser avec Attribute infos !!
			
			int methActi = toAction(methName) ;
			
			MBeanOperationInfo mboi = new MBeanOperationInfo(methName, methDesc, paramNfos, methType, methActi) ;
			
			operNfos.add(mboi) ;
			Log.info("\t\t -> register method as MBeanOperation: "+methName) ;
			
			notfInfos.add(new MBeanNotificationInfo(new String[]{MethodCallNotification.METHOD_CALL}, MethodCallNotification.class.getName(), "Method call")) ;
		}
		
		return operNfos.toArray(new MBeanOperationInfo[operNfos.size()]) ;
	}
	
	final static private int toAction(String methName)
	{
		if(methName.startsWith("get") || methName.startsWith("is") || methName.startsWith("has"))
			return MBeanOperationInfo.INFO ;
		
		if(methName.startsWith("set")) return MBeanOperationInfo.ACTION ;
		
		return MBeanOperationInfo.ACTION_INFO ; // rem : if UNKNOWN then the method cannot be called in JMX Console
	}
	
	
	final static private MBeanAttributeInfo[] computeAttributeInfo(List<MBeanNotificationInfo> notfInfos, Field[] fields)
	throws Exception
	{
		if(fields==null) return null ;
		
		System.out.println("\t\t (*) ComputeAttributeInfo") ;
		
		List<MBeanAttributeInfo> attrNfos = new ArrayList<MBeanAttributeInfo>() ;
		
		try
		{
			for(Field field : fields)
			{
				String fieldName = field.getName() ;
				String fieldDesc = fieldName ; //FIXME: Annotation/Javadocs ?
				String fieldType = field.getType().getName() ;
				
				//TODO: determiner writable, readeable, isIs (attention: getters/setters)
				attrNfos.add(new MBeanAttributeInfo(fieldName, fieldType, fieldDesc, true, true, false)) ;
				
				// add notification definition...
				notfInfos.add(new MBeanNotificationInfo(new String[]{AttributeChangeNotification.ATTRIBUTE_CHANGE}, AttributeChangeNotification.class.getName(),"Attribute "+fieldName+" changed")) ;
				
				Log.info("\t\t\t -> register field as MBeanAttribute: "+fieldName);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		return attrNfos.toArray(new MBeanAttributeInfo[attrNfos.size()]) ;
	}
	
}