/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.jmx.server.imp.console;

import java.util.*;

import javax.management.*;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.*;

import org.azuki.bean.jmx.JMXExportBean;
import org.azuki.bean.jmx.server.JMXExportAdvice;
import org.azuki.bean.jmx.server.imp.JMXUtil;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.Table;
import org.azuki.common.console.http.HttpRequest;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

public class JMXExportConsole
{
	final static public String INFO = "info" ;
	final static public String PARAM_BEAN_ID = "beanID" ;
	
	final static public void service(JMXExportBean asp, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String ruri = req.getRequestURI() ;
		
		printList(asp, req, res) ;
		if(ruri.endsWith(INFO)) printInfo(asp, req, res) ;
	}
	
	final static public void printList(JMXExportBean asp, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		IMetadata meta = Bean.getMetadata(asp) ;
		
		StringBuilder bdr = new StringBuilder() ;
		
		Collection<IPointcut> pcuts = Bean.getPointcuts(meta.getID()) ;
		for (IPointcut pcut : pcuts)
		{
			JMXExportAdvice adv = (JMXExportAdvice)pcut.getAdvice() ;
			if(adv==null) continue ;
			
			String onam = adv.getObjectName().toString() ;
			bdr.append("<a href=\"").append(mkInfoURL(pcut)).append("\">").append(onam).append("</a>") ;
			bdr.append("<br/>") ;
		}
		
		ServletOutputStream sos = res.getOutputStream() ;
		
		sos.println("<h2>Registered MBeans :</h2>") ;
		sos.println(bdr.toString()) ;
		sos.println("<hr size=\"1\"/>") ;
	}
	
	final static public void printInfo(JMXExportBean asp, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String pbid = HttpRequest.get(req, PARAM_BEAN_ID) ;
		Integer bid = Integer.valueOf(pbid) ;
		
		IMetadata meta = Bean.getMetadata(asp) ;
		IMetadata tmeta = Bean.getMetadata(bid) ;
		
		List<IPointcut> lpc = tmeta.getPointcuts(meta.getID()) ;
		if(lpc.isEmpty()) return ;
		
		// Normally applied only once
		IPointcut pcut = lpc.get(0) ;
		
		// properties
		Map<String,String> props = new Hashtable<String,String>() ;
		props.put("ID", Util.oneString(tmeta.getName(), tmeta.getLabel())) ;
		props.put("Description", tmeta.getDescription()) ;
		props.put("Universe > Schema", tmeta.getUniverse()+" > "+tmeta.getSchema()) ;
		// No more props.put("Run-level", String.valueOf(tmeta.getRunLevel())) ;
		
		Table propTable = HtmlFactory.propertyList("Bean metadata", props) ;
		
		// mbean info
		MBeanInfo mbi = ((JMXExportAdvice)pcut.getAdvice()).getMBeanInfo() ;
		
		List<List> attrs = getAttributes(mbi) ;
		List<List> opers = getOperations(mbi) ;
		List<List> notfs = getNotifications(mbi) ;
		
		Table attrTable = HtmlFactory.sortedTable(req, "attrs", attrs, attributeHeaders) ;
		Table operTable = HtmlFactory.sortedTable(req, "opers", opers, operationHeaders) ;
		Table notfTable = HtmlFactory.sortedTable(req, "notfs", notfs, notificationHeaders) ;
		
		// write page
		ServletOutputStream sos = res.getOutputStream() ;
		sos.println("<br />") ;
		sos.println(propTable.toString()) ;
		sos.println("<br />") ;
		sos.print("<h3>Attributes</h3>") ;
		sos.println(attrTable.toString()) ;
		sos.println("<br />") ;
		sos.print("<h3>Operations</h3>") ;
		sos.println(operTable.toString()) ;
		sos.println("<br />") ;
		sos.print("<h3>Notifications</h3>") ;
		sos.println(notfTable.toString()) ;
	}
	
	final static public String[] attributeHeaders = {"Name", "Type", "Description", "Rights"} ;
	final static public List<List> getAttributes(MBeanInfo mbi)
	{
		List<List> list = new ArrayList<List>() ;
		
		for(MBeanAttributeInfo mbai : mbi.getAttributes())
		{
			String name = mbai.getName() ;
			String type = mbai.getType() ;
			String desc = mbai.getDescription() ;
			
			String rights = "" ;
			if(mbai.isReadable()) 	rights+="r" ;
			if(mbai.isWritable()) 	rights+="w" ;
			if(mbai.isIs()) 		rights+="i" ;
			
			list.add(ListUtil.asList(name, type, desc, rights)) ;
		}
		
		return list ;
	}
	
	final static public String[] operationHeaders = {"Signature", "Description", "Impact"} ;
	final static public List<List> getOperations(MBeanInfo mbi)
	{
		List<List> list = new ArrayList<List>() ; 
		
		for(MBeanOperationInfo mboi : mbi.getOperations())
		{
			String stur = JMXUtil.toSignature(mboi) ;
			String desc = mboi.getDescription() ;
			String impc = JMXUtil.getImpactString(mboi.getImpact()) ;
			
			list.add(ListUtil.asList(stur, desc, impc)) ;
		}
		
		return list ;
	}
	
	final static public String[] notificationHeaders = {"Name", "Types", "Description"} ;
	final static public List<List> getNotifications(MBeanInfo mbi)
	{
		List<List> list = new ArrayList<List>() ;
		
		for(MBeanNotificationInfo mbni : mbi.getNotifications())
		{
			String name = mbni.getName() ;
			String typs = StringUtil.toString(mbni.getNotifTypes(), ", ") ;
			String desc = mbni.getDescription() ;
			
			list.add(ListUtil.asList(name, typs, desc)) ;
		}
		
		return list ;
	}
	
	final static private String mkInfoURL(IPointcut pc)
	{
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(INFO).append('?') ;
		bdr.append(PARAM_BEAN_ID).append('=').append(pc.getTargetMetadata().getID()) ;
		
		return bdr.toString() ;
	}
}
