/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.jndi.server.console ;

import java.util.*;

import javax.naming.*;
import javax.servlet.http.*;

import org.azuki.bean.jndi.JNDIResourceBean;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.*;
import org.azuki.common.console.http.*;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class JNDIResourceConsole
{
	final static public String PARAM_ACTION = "action" ;
	
	final static public String ACTION_DUMP = "dump" ;
	
	final static public String[] addrTableHeaders = {"Type", "Value"} ;
	final static public String[] pcutTableHeaders = {"Bean name"} ;
	
	final static public void service(JNDIResourceBean aspt, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String actn = HttpRequest.get(req, PARAM_ACTION) ;
		
		doConfiguration(aspt, srv, req, res) ;
		if(ACTION_DUMP.equals(actn))	doDump(srv, req, res) ;							
	}
	
	final static private void doConfiguration(JNDIResourceBean aspt, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		// properties
		Map<String,String> props = new TreeMap<String,String>() ;
		props.put("Class name", aspt.getClassName()) ;
		props.put("Factory class name", aspt.getFactoryClassName()) ;
		props.put("Factory location", aspt.getFactoryLocation()) ;
		props.put("Context binding", aspt.getContextBinding()) ;
		props.put("Context binding value", aspt.getName()) ;
		props.put("Namespace", aspt.getNamespace()) ;
		
		Table propTable = HtmlFactory.propertyList("Properties", props) ;
		
		// addresses
		List<List> addrs = new ArrayList<List>() ;
		for(Map.Entry<String,String> ety : aspt.getAddresses().entrySet())
			addrs.add(ListUtil.asList(ety.getKey(), ety.getValue())) ;
		
		Table addrTable = HtmlFactory.sortedTable(req, addrs, addrTableHeaders) ;
		
		// pointcuts
		List<List> data = new ArrayList<List>() ;
		
		IMetadata meta = Bean.getMetadata(aspt) ;
		
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			IMetadata tmeta = pc.getTargetMetadata() ; 
			data.add(ListUtil.toList(Util.oneString(tmeta.getName(), tmeta.getLabel()))) ;
		}
		
		Table pcutTable = HtmlFactory.sortedTable(req, data, pcutTableHeaders) ;
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		bdr.print("<h2>").print(Util.oneString(meta.getName(), meta.getLabel())).println("</h2>") ;
		bdr.println("<br/>") ;
		bdr.println(propTable) ;
		bdr.println("<br/>") ;
		bdr.print("<h3>Addresses</h3>") ;
		bdr.println(addrTable) ;
		bdr.println("<br/>") ;
		bdr.print("<h3>Pointcuts</h3>") ;
		bdr.println(pcutTable) ;
		bdr.println("<br/>") ;
		bdr.println("<br/>") ;
		bdr.print("<a href=\"").print(mkDumpURL()).print("\"><button>Dump JNDI contexts</button></a>") ;
		bdr.println("<br/>") ;
		
		HttpResponse.print(res, bdr) ;
	}
	
	final static private void doDump(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		bdr.println("<h1>Useful routines for debugging naming problems</h1>") ;
		
		InitialContext ic = new InitialContext() ;
		
		// environment
		bdr.println("<br/>") ;
		bdr.println("<h2>What naming environment is actually getting used ? : </h2>") ;
		
		Hashtable<?,?> ht = ic.getEnvironment() ;
		for(Map.Entry<?,?> ety : ht.entrySet())
		{
			bdr.print(ety.getKey()).print("=").println(ety.getValue()) ;
			bdr.println("<br/>") ;
		}
		
		bdr.println("<br/>") ;
		bdr.println("<h2>What is actually bound in that context ?</h2>") ;
		
//		bdr.println(dumpNamespace("Global Namespace", 		"")) ;
		bdr.println(dumpNamespace("Java Namespace", 		"java:/")) ;
		bdr.println(dumpNamespace("Component Namespace", 	"java:/comp")) ;
		bdr.println(dumpNamespace("Env Namespace", 			"java:/comp/env")) ;
		bdr.println(dumpNamespace("JDBC", 					"java:/comp/env/jdbc")) ;
		bdr.println(dumpNamespace("EJB refs", 				"java:/comp/env/ejb")) ;

		bdr.println("<br/>") ;
		bdr.println("<h2>Is it actually using my jndi.properties ? Maybe there is more than one ?</h2>") ;
		ClassLoader loader = Thread.currentThread().getContextClassLoader() ;
		for(Enumeration e = loader.getResources("jndi.properties"); e.hasMoreElements();)
		{
			bdr.println(e.nextElement().toString()) ;
			bdr.println("<br/>") ;
		}
		
		HttpResponse.print(res, bdr) ;
	}
	
	final static private String dumpNamespace(String labl, String nspc)
	throws Exception
	{
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		try
		{
			InitialContext ic = new InitialContext() ;
	
			bdr.println("<br/>") ;
			bdr.print("<b>").print(labl).print(" : \"").print(nspc).print("\"</b>") ;
			bdr.println("<br/>") ;
			
			NamingEnumeration ne = ic.listBindings(nspc) ;
			while(ne.hasMore())
			{
				bdr.println(ne.next().toString()) ;
				bdr.println("<br/>") ;
			}
		}
		catch(Exception e)
		{
			bdr.print("<i>").print(e.getMessage()).println("</i>");
			bdr.println("<br/>") ;
		}
		
		return bdr.toString() ;
	}
	
	final static private String mkDumpURL()
	{
		StringBuilder bdr = new StringBuilder() ;
		bdr.append('?') ;
		bdr.append(PARAM_ACTION).append('=').append(ACTION_DUMP) ;
		
		return bdr.toString() ;
	}
}