/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.jpa  ;

import java.util.Map;

import javax.naming.NamingException;
import javax.persistence.*;

import org.azuki.bean.jpa.server.imp.EntityManagerAdvice;
import org.azuki.bean.jpa.shared.ConstTransaction;
import org.azuki.common.editor.EditorUtil;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.context.*;
import org.azuki.framework.object.*;
import org.jdom.Element;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class EntityManagerBean implements IAspect<EntityManagerAdvice>, IObjectFactory<EntityManager>, ILifecycleFactory<EntityManager>
{
	public enum DTAction { NOTHING, COMMIT, ROLLBACK }
	
	// Lazy
	private EntityManagerFactory emf = null ;
	
	
	private String PUN = null ; // Persistence unit name
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception 
	{
		Map<String, String> map = EditorUtil.formToMap(para) ;
    	if (map==null)
    		throw new Exception("ERROR : No parameter found");

		PUN = map.get("pun") ;
		if (PUN==null || PUN.isEmpty())
    		throw new Exception("ERROR : Persistence Unit Name parameter is not set") ;		
	}	
	
	@MethodInfo(desc="Should be called at shutdown to close entitymanager")
	final public void azukiStopped()
	{		
		if(emf!=null && emf.isOpen()) emf.close();
	}
	
	
	@Override
	final public EntityManagerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		Element paraLmen = ipc.getParameter() ;
		
		Element tranLmen = JDOMUtil.getElement(paraLmen, ConstTransaction.TRAN) ;
		
		String bforStr = tranLmen.getAttributeValue(ConstTransaction.TRAN_BFOR, DTAction.NOTHING.name()) ;
		String aftrStr = tranLmen.getAttributeValue(ConstTransaction.TRAN_AFTR, DTAction.NOTHING.name()) ;
		String erroStr = tranLmen.getAttributeValue(ConstTransaction.TRAN_ERRO, DTAction.NOTHING.name()) ;
		
		DTAction bfor = DTAction.NOTHING ;
		try
		{ bfor = DTAction.valueOf(bforStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid data transaction before action: "+bforStr+" using default.") ; }
		
		DTAction aftr = DTAction.NOTHING ;
		try
		{ aftr = DTAction.valueOf(aftrStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid data transaction after action: "+aftrStr+" using default.") ; }
		
		DTAction erro = DTAction.NOTHING ;
		try
		{ erro = DTAction.valueOf(erroStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid data transaction on error action: "+erroStr+" using default.") ; }
		
		return new EntityManagerAdvice(this, bfor, aftr, erro);
	}
	
	
// Note BOB : Afin d'eviter la multiplication des instances (et notament de devoir instancier le factory sous forme d'un singleton dans le kernel)
// La fonction factory est implementé dans le bean entity manager.
// Cela a pour conséquence que chaque instance de EntityManagerBean va se declarer comme la factory du PNU qui lui est associé.
// De plus, cela est necessaire pour conserver les EMFactory
	final public String getObjectName()
	{
		return new StringBuilder(IContext.OBJECT_ENTITYMANAGER).append('_').append(PUN).toString() ;
	}
	
	// Datasource can be passed as parameter or found in "dataource" context variable.
	final public EntityManager createObject(Object... args)
	throws Exception
	{
		return createEntityManager() ;		
	}	
	
    final public ILifecycle<EntityManager> getLifeCycle()
	{
		return new TransactionLifecycle() ;
	}
	
    class TransactionLifecycle implements ILifecycle<EntityManager>
    {
    	final public EntityManager fork(EntityManager em)
    	{
    		return null ; // EntityManager cannot be shared between multiple thread
    	}

    	final public void end(EntityManager em)
    	{
   	   		if(em!=null  && em.isOpen()) em.close();
    	}
    }   	


    @Export
	final public String getProcessUnitName()
    {
    	return PUN ;
    }
    
    
	@Export
	@MethodInfo(desc="Create a new entity manager instance")
	final public EntityManager createEntityManager()
	throws Exception
	{
		try
		{
			if (PUN==null || PUN.isEmpty())
				throw new NamingException("Undefined Persistence Unit Name");

			if(emf==null) emf = Persistence.createEntityManagerFactory(PUN);
			return emf.createEntityManager();
		}
		catch(Throwable thr)
		{
			Log.severe(thr) ;
			throw new Exception(thr) ;
		}
	}
}
