/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.queuing ;

import java.util.Collection;

import javax.servlet.http.*;

import org.azuki.bean.queuing.server.QueuingAdvice;
import org.azuki.bean.queuing.server.console.QueuingConsole;
import org.azuki.bean.queuing.server.shell.QueuingShell;
import org.azuki.bean.queuing.shared.ConstQueuing;
import org.azuki.common.util.Util;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

// Cet aspect doit etre associé a un bean qui implement l'interface IEventListener
//---
// Sample pointcut
//
// <pointcut name="Queuing" labl="" enab="yes">
//   <para>
//     <queue thread="10" size="100" shift="no" />  
//   </para>
// </pointcut>
//

final public class QueuingBean implements IAspect<QueuingAdvice>
{
	private IMetadata meta = null ;
	public IMetadata getMetadata()
	throws Exception
	{ 
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	final private QueuingAdvice findAdvice(String name, String labl)
	throws Exception
	{
		for(IPointcut pc : Bean.getPointcuts(getMetadata().getID()))
		{
			IMetadata tmeta = pc.getTargetMetadata() ;
			
			if(name!=tmeta.getName())	continue ;
			if(labl!=tmeta.getLabel())	continue ;
			
			return (QueuingAdvice)pc.getAdvice() ;
		}
		return null ;
	}
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public QueuingAdvice createAdvice(IPointcut ipc)
	{
		IMetadata meta = ipc.getTargetMetadata() ;
		
		// Parameters
		int thrd = 1 ;
		int size = 99999999 ;
		boolean shft = false ;
		
		Element para = ipc.getParameter() ;
		if(para==null) return new QueuingAdvice(meta, thrd, size, shft) ;
		
		Element queuLmen = para.getChild(ConstQueuing.QUEUE) ;
		if(queuLmen==null) return new QueuingAdvice(meta, thrd, size, shft) ;
		
		String thrdStr = queuLmen.getAttributeValue(ConstQueuing.THREAD) ;
		try
		{ thrd = Integer.parseInt(thrdStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid thread number value ("+thrdStr+"), using default") ; }
		
		String sizeStr = queuLmen.getAttributeValue(ConstQueuing.SIZE) ;
		try
		{ size = Integer.parseInt(sizeStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid size value ("+sizeStr+"), using default") ; }
		
		String shftStr = queuLmen.getAttributeValue(ConstQueuing.SHIFT) ;
		if(shftStr!=null) shft = Util.isOK(shftStr) ;
		
		return new QueuingAdvice(meta, thrd, size, shft) ;
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		QueuingConsole.service(this,servlet,req,res);
	}
	
	@MethodInfo(desc="Build the list of all QueuingBean advice",
				retu="The advices listing")
	final public String listQueues()
	throws Exception
	{
		return QueuingShell.listPointcuts(this) ;
	}
	
	@MethodInfo(desc="Return the beans's queuing informations",
				retu="The information text")	
	final public String getQueueInfo(@ArgInfo(name="bean", desc="the bean's name:label") String bean)
	throws Exception
	{
		Util.Key bkey = Util.Key.valueOf(bean) ;
		
		QueuingAdvice advc = findAdvice(bkey.getName(), bkey.getLabel()) ;
		if(advc==null) throw new Exception("Queuing advice not found: "+bean) ;
		
		return QueuingShell.getAdviceInfo(advc) ;
	}
	
	final public int getMaxThread(@ArgInfo(name="bean", desc="the bean's name:label") String bean)
	throws Exception
	{
		Util.Key bkey = Util.Key.valueOf(bean) ;
		
		QueuingAdvice advc = findAdvice(bkey.getName(), bkey.getLabel()) ;
		if(advc==null) throw new Exception("Queuing advice not found: "+bean) ;
		
		return advc.getMaxThread() ;
	}
	
	final public void setMaxThread(@ArgInfo(name="bean", desc="the bean's name:label")String bean,
								   @ArgInfo(name="maxt", desc="the new max thread value") int maxt)
	throws Exception
	{
		Util.Key bkey = Util.Key.valueOf(bean) ;
		
		QueuingAdvice advc = findAdvice(bkey.getName(), bkey.getLabel()) ;
		if(advc==null) throw new Exception("Queuing advice not found: "+bean) ;
		
		advc.setMaxThread(maxt) ;
	}
	
	final public int getQueueSize(@ArgInfo(name="bean", desc="the bean's name:label") String bean)
	throws Exception
	{
		Util.Key bkey = Util.Key.valueOf(bean) ;
		
		QueuingAdvice advc = findAdvice(bkey.getName(), bkey.getLabel()) ;
		if(advc==null) throw new Exception("Queuing advice not found: "+bean) ;
		
		return advc.getQueueSize() ;
	}
	
	final public void setQueueSize(@ArgInfo(name="bean", desc="the bean's name:label") String bean,
								   @ArgInfo(name="size", desc="the queue size") int size)
	throws Exception
	{
		Util.Key bkey = Util.Key.valueOf(bean) ;
		
		QueuingAdvice advc = findAdvice(bkey.getName(), bkey.getLabel()) ;
		if(advc==null) throw new Exception("Queuing advice not found: "+bean) ;
		
		advc.setQueueSize(size) ;
	}
	
	//--
	
	final public String[][] getQueues()
	throws Exception
	{
		Collection<IPointcut> pcs = Bean.getPointcuts(getMetadata().getID()) ;
		
		String[][] out = new String[pcs.size()][3] ;
	
		int i = 0 ;
		for(IPointcut pc : pcs)
		{
			QueuingAdvice adv = (QueuingAdvice)pc.getAdvice() ;
			if(adv==null) continue ;
			
			out[i][0] = pc.getTargetMetadata().getID().toString() ;
			out[i][1] = String.valueOf(adv.getQueueSize()) ;
			out[i][2] = String.valueOf(adv.getMaxThread()) ;
			
			i++ ;
		}
		
		return out ;
	}
}