/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.scheduler ;

import java.lang.reflect.Method;
import java.util.List;

import javax.servlet.http.*;

import org.azuki.bean.scheduler.server.SchedulerAdvice;
import org.azuki.bean.scheduler.server.console.SchedulerConsole;
import org.azuki.bean.scheduler.server.imp.SchedulerImp;
import org.azuki.bean.scheduler.server.shell.SchedulerShell;
import org.azuki.common.util.Util;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.command.ICommand;
import org.jdom.Element;


final public class SchedulerBean implements IAspect<SchedulerAdvice> 
{
	// Si aucune methode n'est selectionnée alors la classe doit implementer ICommand
	// Sinon, appel de la methode selectionné qui ne doit pas avoir d'arguments
/*	
    <!-- default parameter for Advice -->
    <para>
		<type  value="PERIODIC" hour="1" minu="0" seco="0" />
		<week  all="yes" days="0000000" />
		<month all="yes" days="0000000000000000000000000000000" />
    </para>
*/
		
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public SchedulerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		Method[] methods = ipc.getSelectedMethods() ;
		
		if(methods.length<1)
			throw new Exception("No method selected") ;

		if(methods.length>1)
			throw new Exception("This aspect is applied to one method only") ;
		
		Method sel = methods[0] ;
	
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		SchedulerAdvice cmd = new SchedulerAdvice(tmeta, sel.getName()) ;
	
		String jobname = Util.oneString(tmeta.getName(), tmeta.getLabel())+"."+sel.getName() ;
		
		scheduleJobElement(jobname, cmd, ipc.getParameter()) ;
		
		return cmd ;
	}

	final public void startAll()
	throws Exception
	{
		if (list()==null || list().isEmpty())
			return;
		List<Object[]> jobList = list();
		for (Object[] job : jobList)
		{
			Integer ID = (Integer) job[0];
			resume(ID);
		}
	}
	
	final public void stopAll()
	throws Exception
	{
		if (list()==null || list().isEmpty())
			return;
		List<Object[]> jobList = list();
		for (Object[] job : jobList)
		{
			Integer ID = (Integer) job[0];
			suspend(ID);
		}
	}

	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		SchedulerConsole.service(this, servlet, req, res) ;
	}
	
	@MethodInfo(desc="Build the list of all jobs scheduled", retu="The jobs listing")
	final public String listSchedulerJobs()
	throws Exception
	{
		return SchedulerShell.listJobs(this) ;
	}
	
	
// Service Part	
//////////////////////////////////////////////////////	

	final private SchedulerImp imp = new SchedulerImp() ;
	
	/**
	 * Schedules a job to be launched for repeated execution at regular intervals of time. 
	 * @param  cmd The command pattern interface
	 * @param period The period in millis
	 * @param hour   The hour
	 * @param minute The minute

	 * @return The scheduling ID of the job
	 * @throws IllegalArgumentException
	 */
	@Export
	final public Integer scheduleJob(String jobName, ICommand icmd, long period, int hour, int minu, int sec, boolean[] weekdays, boolean[] monthdays)
	throws IllegalArgumentException
	{
	    return imp.scheduleJob(jobName, icmd, period, hour, minu,sec, weekdays, monthdays) ;
	}

	
	/**
	 * Schedules a job to be launched for repeated execution at regular intervals of time. 
	 * @param  cmd The command pattern interface
	 * @param  para Xml parameters
	 * @return The scheduling ID of the job
	 * @throws IllegalArgumentException
	 */
	@Export
	final public Integer scheduleJobElement(String jobName, ICommand icmd,  Element para)
	throws IllegalArgumentException
	{
	    return imp.scheduleJob(jobName, icmd, para) ;
	}
	
	
	/**
	 * Schedules a job to be launched only once, after the <i>delai</i> has passed.
	 * @param job The job
	 * @param delai The delai to wait before launching the job
	 * @return The scheduling ID of the job
	 */
	@Export
    final public Integer sendJobAfter(String jobName, ICommand icmd, long delai)
    {
    	return imp.sendJobAfter(jobName, icmd, delai) ;  
    }

    /**
     * Cancels the scheduling of the job.<br>
     * The job is not relaunched once this method has been called.
     * @param ID The scheduling ID of the job.
     */
	@Export
	final public void cancel(Integer ID) // Implementation
	{
	    imp.cancel(ID);
	    
	}
	
	/**
	 * Suspend job execution but don't cancel it
	 * @param ID The scheduling ID of the job.
	 * @param Suspend : true to suspend job execution, false to activate the job
	 */
	@Export
	final public void suspend(Integer ID)
	{
		imp.suspend(ID, true);
	}

	/**
	 * Resume job execution but don't cancel it
	 * @param ID The scheduling ID of the job.
	 */
	@Export
	final public void resume(Integer ID)
	{
		imp.suspend(ID, false);
	}

	// Not exported
	final public List<Object[]> list()
	{
		return imp.list();
	}
}