/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.snmp ;

import java.util.*;

import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.*;
import org.snmp4j.*;
import org.snmp4j.mp.*;
import org.snmp4j.smi.*;
import org.snmp4j.transport.*;



/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

/*
 * Convert a SNMP Trap to Azuki's event
	// Prefix: iso.org.dod.internet.private.enterprise (1.3.6.1.4.1)
	// Azuki : 29282
 */

final public class SNMPTrapReceiverBean implements CommandResponder
{
	final static private String outputName = "xmlpdu";
	
	// EventPublisher injection
	@DependencyInjectionInfo(desc="Publish a message to an output connector",
							param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish ;
	
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element lmen)
	throws Exception
	{
		// TODO : mk adresse with parameter
		String protocol = "udp" ;
		String addr     = "0.0.0.0" ; // ?

		// Parameters
		Map<String, String> map = EditorUtil.formToMap(lmen) ;
    	if (map==null)
    		throw new Exception("ERROR : No parameter found");

		String sport = map.get("port") ;
		if (sport==null)
    		throw new Exception("ERROR : Port parameter is not set") ;

		int port = 162 ; // SNMP default
		try
    	{
    		port = Integer.parseInt(sport) ;
    	}
    	catch(NumberFormatException nfe)
    	{
    		Log.severe("Listening port number format exception ("+sport+")") ;
    		throw nfe ;
    	}
		
		
		String fulladdr = new StringBuilder(protocol).append(":").append(addr).append("/").append(port).toString() ;
		
		
		Address listenAddress = GenericAddress.parse(fulladdr);

		TransportMapping transport;
	    if (listenAddress instanceof UdpAddress)
	    	transport = new DefaultUdpTransportMapping((UdpAddress)listenAddress);
	    else
	    	transport = new DefaultTcpTransportMapping((TcpAddress)listenAddress);
	 
	    Snmp snmp = new Snmp(transport);
		
	    MessageDispatcher md = snmp.getMessageDispatcher() ;
	    md.addMessageProcessingModel(new MPv1());
	    md.addMessageProcessingModel(new MPv2c());
	    md.addMessageProcessingModel(new MPv3());
	    
	    // ?
	    /*
	    USM usm = new USM(SecurityProtocols.getInstance(), new OctetString(MPv3.createLocalEngineID()), 0);
	    SecurityModels.getInstance().addSecurityModel(usm);
	     */
	    snmp.addCommandResponder(this);
	      
	    snmp.listen();
	}	
	
	
	// Interface CommandResponder
	@Override
	final public void processPdu(CommandResponderEvent cre)
	{
		Document event = toXMLDocument(cre.getPDU()) ;
		
		try
		{
			if(publish==null) throw new Exception("Method publish() needs to be injected") ;
			
			publish.call(this, outputName, event) ;
		} 
		catch (Exception e)
		{
			Log.severe(e) ;
		}
	}
	
	
	// <pdu type="" request="" errstat="" erridx="" >
	//     <vb oid="" var="" />
	//     ...
	// </pdu>
	
	@SuppressWarnings("unchecked")
	final static public Document toXMLDocument(PDU pdu)
	{		
		String styp = PDU.getTypeString(pdu.getType()) ;
		String reqid =String.valueOf(pdu.getRequestID()) ;
		String errstat = pdu.getErrorStatusText() ;
		String erridx  = String.valueOf(pdu.getErrorIndex()) ;
		
		Document doc = new Document() ;
		
		
		Element lmen = new Element("pdu") ;
		lmen.setAttribute("type", styp) ;
		lmen.setAttribute("request", reqid) ;
		lmen.setAttribute("errstat", errstat) ;
		lmen.setAttribute("erridx", erridx) ;

		Vector<VariableBinding> vvb = pdu.getVariableBindings() ;
		for(VariableBinding vb : vvb)
		{
			Element child = new Element("vb") ;
			child.setAttribute("oid", String.valueOf(vb.getOid())) ;
			child.setAttribute("var", String.valueOf(vb.getVariable())) ;
			lmen.addContent(child) ;
		}
		
		/*
		  public String toString()
		  {
			    StringBuffer buf = new StringBuffer();
			    buf.append(getTypeString(type));
			    buf.append("[requestID=");
			    buf.append(requestID);
			    buf.append(", errorStatus=");
			    buf.append(getErrorStatusText()+"("+errorStatus+")");
			    buf.append(", errorIndex=");
			    buf.append(errorIndex);
			    buf.append(", VBS[");
			    for (int i=0; i<variableBindings.size(); i++) {
			      buf.append(variableBindings.get(i));
			      if (i+1 < variableBindings.size()) {
			        buf.append("; ");
			      }
			    }
			    buf.append("]]");
			    return buf.toString();
			  }
		*/
		
		doc.addContent(lmen) ;
		return doc ;
	}
}