/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.weaver ;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.List;
import java.util.logging.*;

import javax.swing.*;
import javax.swing.event.*;

import org.azuki.common.editor.IPointcutEditorContext;
import org.azuki.common.reflect.*;
import org.azuki.common.widget.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class MethodProxySelector extends JDialogPanel implements ListSelectionListener, MouseListener 
{
	final static protected Font descAreaFont = UIManager.getFont("Label.font").deriveFont(Font.PLAIN).deriveFont(Font.ITALIC) ;
	
	final protected IPointcutEditorContext pctx ;
	final protected String fnam ;
	
	final private Field proxyField ;
	
	final private Signature methodSignature ;
	
	protected JLabel proxyLabel ;
	protected JLabel proxyNameLabel ;
	protected JTextArea proxyDesc ;
	protected JScrollPane proxyDescScroll ;
	protected JPanel proxyPanel ;
	
	protected JFilteredList<BeanInfo> beanInfoList ;

	protected JGenericList<String> beanLabelList;
	
	protected JGenericList<Signature> beanMethodList;
	protected JTextArea beanMethodDesc ;
	protected JScrollPane beanMethodDescScroll ;
	protected JPanel beanMethodPanel ;
	
	protected JPanel topPanel ;
	protected JPanel rightPanel ;
	
	public MethodProxySelector(IPointcutEditorContext pctx, String fnam)
	{
		this.pctx = pctx ;
		this.fnam = fnam ;
		
		Field field = null ;
		try
		{ field = pctx.getTargetInfo().getClassImpl().getField(fnam) ; }
		catch(Exception ex)
		{ Logger.getAnonymousLogger().log(Level.SEVERE, "Proxy field not found", ex) ; }
		finally
		{ this.proxyField = field ; }
		
		Signature stur = null ;
		try
		{ stur = new Signature(BeanReflect.getMethodProxySignature(proxyField)) ; }
		catch(Exception ex)
		{ Logger.getAnonymousLogger().log(Level.SEVERE, "Cannot parse field signature", ex) ; }
		finally
		{ this.methodSignature = stur ; }
		
		if(methodSignature==null) return ;

		// Proxy label
		proxyNameLabel = new JLabel(methodSignature.getSignature()) ;
		proxyNameLabel.setFont(proxyNameLabel.getFont().deriveFont(Font.BOLD)) ;
		
		proxyLabel = new JLabel("Method proxy :") ;
		proxyLabel.setFont(proxyLabel.getFont().deriveFont(Font.BOLD)) ;
		proxyLabel.setLabelFor(proxyNameLabel) ;
		
		proxyDesc = new JTextArea() ;
		proxyDesc.setEditable(false) ;
		proxyDesc.setFont(descAreaFont) ;
		proxyDesc.setLineWrap(true) ;
		proxyDesc.setRows(3) ;
		proxyDesc.setOpaque(false) ;
		if(proxyField!=null) proxyDesc.setText(Annotations.getDependencyInjectionDescription(proxyField)) ;
		
		proxyDescScroll = new JScrollPane(proxyDesc) ;
		proxyDescScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS) ;
		proxyDescScroll.setBorder(BorderFactory.createEmptyBorder()) ;
		
		proxyPanel = new JPanel(new BorderLayout()) ;
		proxyPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), BorderFactory.createEmptyBorder(2,2,2,2))) ;
		proxyPanel.add(proxyNameLabel, BorderLayout.CENTER) ;
		proxyPanel.add(proxyDescScroll, BorderLayout.SOUTH) ;
		
		topPanel = new JPanel(new BorderLayout()) ;
		topPanel.add(proxyLabel, BorderLayout.NORTH) ;
		topPanel.add(proxyPanel, BorderLayout.SOUTH) ;
		
		// Bean info list
		List<BeanInfo> beans = new ArrayList<BeanInfo>(pctx.getBeanInfos()) ;
		Collections.sort(beans, new BeanInfoComparator(methodSignature)) ;
		
		beanInfoList = new JFilteredList<BeanInfo>(beans) ;
		beanInfoList.setCellRenderer(new BeanInfoRenderer(methodSignature)) ;
		beanInfoList.setPreferredSize(new Dimension(200, 200)) ;
		
		beanInfoList.getJList().addMouseListener(this) ;
		beanInfoList.getJList().addListSelectionListener(this) ;
		
		// Bean label selection
		beanLabelList = new JGenericList<String>() ;
		beanLabelList.setBorder(BorderFactory.createTitledBorder("Label")) ;
		beanLabelList.setCellRenderer(new BeanLabelRenderer()) ;
		
		beanLabelList.getJList().addListSelectionListener(this) ;
		
		// Bean method selection
		beanMethodList = new JGenericList<Signature>() ;
		beanMethodList.setBorder(BorderFactory.createEmptyBorder()) ;
		beanMethodList.setCellRenderer(new SignatureRenderer(methodSignature)) ;
		
		beanMethodList.getJList().addListSelectionListener(this) ;
		
		beanMethodDesc = new JTextArea() ;
		beanMethodDesc.setEditable(false) ;
		beanMethodDesc.setFont(descAreaFont) ;
		beanMethodDesc.setLineWrap(true) ;
		beanMethodDesc.setOpaque(false) ;
		beanMethodDesc.setRows(3) ;
		
		beanMethodDescScroll = new JScrollPane(beanMethodDesc) ;
		beanMethodDescScroll.setBorder(BorderFactory.createEmptyBorder()) ;
		
		beanMethodPanel = new JPanel(new BorderLayout()) ;
		beanMethodPanel.setBorder(BorderFactory.createTitledBorder("Method")) ;
		beanMethodPanel.add(beanMethodList, BorderLayout.CENTER) ;
		beanMethodPanel.add(beanMethodDescScroll, BorderLayout.SOUTH) ;
		
		// Right panel
		rightPanel = new JPanel() ;
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS)) ;
		rightPanel.add(beanLabelList) ;
		rightPanel.add(beanMethodPanel) ;
		
		// Main panel
		setLayout(new BorderLayout(5,5)) ;
		add(topPanel, BorderLayout.NORTH) ;
		add(beanInfoList, BorderLayout.WEST) ;
		add(rightPanel, BorderLayout.CENTER) ;
		
		setPreferredSize(new Dimension(512,384)) ;
	}
	
	final public void setSelection(String name, String labl, String meth)
	{
		// select bean name
		BeanInfo bnfo = pctx.getBeanInfo(name) ;
		if(bnfo==null) return ;
		
		beanInfoList.setSelectedItem(bnfo) ;
		
		// select bean label
		beanLabelList.setSelectedItem(labl) ;
		
		// select bean method
		Method m = null ;
		try{ m = ReflectUtil.getMethod(bnfo.getClassImpl(), meth) ; }
		catch(Exception ex){}
		if(m==null) return ;
		
		beanMethodList.setSelectedItem(new Signature(m)) ;
	}
	
	final public boolean isSelectionValid()
	{
		if(beanInfoList.getSelectedItem()==null) 	return false ;
		if(beanLabelList.getSelectedItem()==null)	return false ;		
		if(beanMethodList.getSelectedItem()==null)	return false ;
		
		return true ;
	}
	
	final public String getSelectedBeanName()
	{
		return beanInfoList.getSelectedItem().getName() ;
	}
	
	final public String getSelectedBeanLabel()
	{
		String labl = beanLabelList.getSelectedItem() ;
		return labl==null ? "" : labl ;
	}
	
	final public String getSelectedMethodName()
	{
		Signature s = beanMethodList.getSelectedItem() ;
		return s==null ? "" : s.getMethod() ;
	}
	
	final private void manageOKButton()
	{
		if(dialog==null) return ;
		
		Action okAction = dialogActions.getAction(JDialogPanel.OK_ACTION) ;
		okAction.setEnabled(isSelectionValid()) ;
	}
	
	/*
	 * JDialogPanel
	 */
	@Override
	final protected void preShow()
	{
		super.preShow() ;
		
		JList list = beanInfoList.getJList() ;
		list.ensureIndexIsVisible(list.getSelectedIndex()) ;
		
		list = beanLabelList.getJList() ;
		list.ensureIndexIsVisible(list.getSelectedIndex()) ;
		
		list = beanMethodList.getJList() ;
		list.ensureIndexIsVisible(list.getSelectedIndex()) ;
		
		manageOKButton() ;
	}
		
	/*
	 * ListSelectionListener
	 */
	final public void valueChanged(ListSelectionEvent e)
	{
		if(e.getValueIsAdjusting()) return ;
		
		Object source = e.getSource() ;
		
		if(source==beanInfoList.getJList())
		{
			BeanInfo bnfo = beanInfoList.getSelectedItem() ;
			if(bnfo==null) // No selection
			{	beanLabelList.getJList().clearSelection() ;
				beanMethodList.getJList().clearSelection() ;
				return ; }
			
			// labels
			List<String> labels = new ArrayList<String>(pctx.getDeclaredInstances(bnfo.getName())) ;
			beanLabelList.setModel(labels) ;
			
			// methods
			List<Signature> sturs = new ArrayList<Signature>(ReflectUtil.getSignatures(bnfo.getClassImpl().getMethods())) ;
			Collections.sort(sturs, new SignatureComparator(methodSignature)) ;
			beanMethodList.setModel(sturs) ;
		}
		else if(source==beanLabelList.getJList())
		{}
		else if(source==beanMethodList.getJList())
		{
			BeanInfo bnfo = beanInfoList.getSelectedItem() ;
			Signature stur = beanMethodList.getSelectedItem() ;
			
			if(bnfo==null || stur==null)
			{
				beanMethodDesc.setText("") ;
			}
			else
			{
				try
				{
					Method method = ReflectUtil.getMethod(bnfo.getClassImpl(), stur.getMethod()) ;
					beanMethodDesc.setText(Annotations.getMethodDescription(method)) ;
					beanMethodDesc.setCaretPosition(0) ;
				}
				catch(Exception ex)
				{ /* TODO */ }
			}
		}
		
		manageOKButton() ;
	}
	
	/*
	 * MouseListener
	 */
	final public void mousePressed(MouseEvent e)
	{
		if(SwingUtilities.isRightMouseButton(e))
		{
			int selIndex = beanInfoList.getJList().locationToIndex(e.getPoint()) ;
			if(selIndex==-1) return ;
			
			beanInfoList.getJList().setSelectedIndex(selIndex) ;
		}
	}

	final public void mouseReleased(MouseEvent e)
	{}
	
	final public void mouseClicked(MouseEvent e)
	{}

	final public void mouseEntered(MouseEvent e)
	{}

	final public void mouseExited(MouseEvent e)
	{}
	
	/**
	 * INNER CLASSES
	 */
	final class BeanInfoRenderer extends DefaultListCellRenderer
	{
		final private Signature stur ;
		
		public BeanInfoRenderer(Signature stur)
		{
			this.stur = stur ;
		}
		
		/*
		 * DefaultListCellRenderer
		 */
		@Override
		final public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			DefaultListCellRenderer c = (DefaultListCellRenderer)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus) ;
			if(!BeanInfo.class.isInstance(value)) return c ;
			
			BeanInfo bnfo = (BeanInfo)value ;
			
			c.setText(bnfo.getName()) ;
			c.setIcon(new ImageIcon(bnfo.getIcon().getImage().getScaledInstance(16, 16, Image.SCALE_SMOOTH))) ;
			
			if(stur==null) return c ;
			
			boolean bold = !BeanReflect.getMethodCandidates(bnfo.getClassImpl(), stur).isEmpty() ;
			boolean gray = BeanReflect.getMethodsExported(bnfo.getClassImpl()).isEmpty() ;
			
			c.setFont(c.getFont().deriveFont(bold ? Font.BOLD : Font.PLAIN)) ;
			c.setForeground(gray ? Color.gray : Color.black) ;
			
			return c ;
		}
	}
	
	final class BeanInfoComparator implements Comparator<BeanInfo>
	{
		final private Signature stur ;
		
		public BeanInfoComparator(Signature stur)
		{
			this.stur = stur ;
		}
		
		/*
		 * Comparator
		 */
		final public int compare(BeanInfo bi1, BeanInfo bi2)
		{
			if(stur==null) return bi1.getName().compareTo(bi2.getName()) ;
			
			boolean b1 = !BeanReflect.getMethodCandidates(bi1.getClassImpl(), stur).isEmpty() ;
			boolean b2 = !BeanReflect.getMethodCandidates(bi2.getClassImpl(), stur).isEmpty() ;
			if(b1!=b2) return b1 ? -1 : 1 ;
			
			b1 = !BeanReflect.getMethodsExported(bi1.getClassImpl()).isEmpty() ;
			b2 = !BeanReflect.getMethodsExported(bi2.getClassImpl()).isEmpty() ;
			if(b1!=b2) return b1 ? -1 : 1 ;
			
			return bi1.getName().compareTo(bi2.getName()) ;
		}
	}
	
	final class BeanLabelRenderer extends DefaultListCellRenderer
	{
		/*
		 * DefaultListCellRenderer
		 */
		@Override
		final public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			DefaultListCellRenderer c = (DefaultListCellRenderer)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus) ;
			
			if(!String.class.isInstance(value)) return c ;
			
			String blbl = (String)value ;
			
			c.setFont(c.getFont().deriveFont(Font.PLAIN)) ;
			if(blbl.isEmpty())
			{	c.setText("(none)") ;
				c.setForeground(Color.gray) ; }
			
			return c ;
		}
	}
	
	final class SignatureRenderer extends DefaultListCellRenderer
	{
		final private Signature stur ;
		
		public SignatureRenderer(Signature stur)
		{
			this.stur = stur ;
		}
		
		/*
		 * DefaultListCellRenderer
		 */
		@Override
		final public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			DefaultListCellRenderer c = (DefaultListCellRenderer)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus) ;
			
			if(!Signature.class.isInstance(value)) return c ;
			if(stur==null) return c ;
			
			Signature sturVal = (Signature)value ;
			
			boolean bold = BeanReflect.isCandidate(stur, sturVal) ;
			c.setFont(c.getFont().deriveFont(bold ? Font.BOLD : Font.PLAIN)) ;
		
			return c ;
		}
	}
	
	final class SignatureComparator implements Comparator<Signature>
	{
		final private Signature stur ;
		
		public SignatureComparator(Signature stur)
		{
			this.stur = stur ;
		}
		
		/*
		 * Comparator
		 */
		final public int compare(Signature s1, Signature s2)
		{
			if(stur==null) return s1.compareTo(s2) ;
			
			boolean b1 = BeanReflect.isCandidate(stur, s1) ;
			boolean b2 = BeanReflect.isCandidate(stur, s2) ;
			
			return b1==b2 ? s1.compareTo(s2) : (b1 ? -1 : 1) ;
		}
	}
}