/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.io.*;
import java.util.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class StringUtil
{
	final static public boolean isEmpty(String str)
	{
		return str==null || str.trim().isEmpty() ;
	}
	
	final static public boolean isNumber(String src)
	{
	    if(src==null) return false ;
	    
	    try					{ Double.parseDouble(src) ; }
	    catch(Exception ex)	{ return false ; }
		
		return true ;	
	}
	
	final static public int[] indexesOf(String src, char c)
	{
		return indexesOf(src, String.valueOf(c)) ;
	}
	
	final static public int[] indexesOf(String src, String s)
	{
		if(src==null) throw new IllegalArgumentException("Source string cannot be null.") ;
		
		if(s==null || s.isEmpty()) return new int[0] ;
		
		int idx = 0 ;
		
		List<Integer> itgs = new ArrayList<Integer>() ;
		
		while(true)
		{
			idx = src.indexOf(s, idx) ;
			if(idx==-1) break ;
			
			itgs.add(new Integer(idx)) ;
			idx++ ;
		}
		
		idx = 0 ;
		
		int[] idxs = new int[itgs.size()] ;
		for(Integer itg : itgs)
			idxs[idx++] = itg.intValue() ;
		
		return idxs ;
	}
	
	final static public String replace(String src, String oldStr, String newStr)
	{
		int len = oldStr.length() ;
		
		int[] idxs = indexesOf(src, oldStr) ;
		
		StringBuilder bdr = new StringBuilder(src) ; 
		
		for(int i=idxs.length-1 ; i>=0 ; i--)
		{	int idx = idxs[i] ;
			bdr.replace(idx, idx+len, newStr) ; }
		
		return bdr.toString() ;
	}
	
	final static public String slashStart(String src)
	{
		return slashed(src, true, false) ;
	}
	
	final static public String slashed(String src, boolean start, boolean end)
	{
		if(src==null) return "/" ;
		
		StringBuilder bdr = new StringBuilder() ;
		if(start) bdr.append('/') ;
		bdr.append(unslashed(src)) ;
		if(end) bdr.append('/') ;
		
		return bdr.toString() ;
	}
	
	final static public String unslashed(String src)
	{
		if(src==null) return null ;
		
		char[] arr = src.toCharArray() ;
		
		int idx1 = 0 ;
		for(int i=0 ; i<arr.length ; i++)
			if(arr[i]!='/') { idx1 = i ; break ; }	
			
			
		int idx2 = -1 ;
		for(int i=arr.length-1 ; i>-1 ; i--)
			if(arr[i]!='/') { idx2 = i ; break ; }
		
		return src.substring(idx1, idx2+1) ;
	}
	
	final static public String nextToken(String src, String sep)
	{
		int idx1 = src.startsWith(sep) ? src.indexOf(sep)+1 : 0 ;
		
		int idx2 = src.indexOf(sep, idx1) ;
		if(idx2==-1) idx2 = src.length() ;
		
		return src.substring(idx1, idx2) ;
	}
	
	final static public String lastToken(String src, String sep)
	{
		int idx = src.lastIndexOf(sep) ;
		return src.substring(idx+1) ;
	}
	
	final static public String trunc(String src, int len)
	{
		return src.length()<len ? src : src.substring(0, len-1) ;
	}
	
	final static public <T extends Object> String toString(T[] arr, String sep)
	{
		if(arr==null) return null ;
		
		StringBuilder bdr = new StringBuilder() ;
		
		boolean first = true ;
		for(T t : arr)
		{
			if(first)	first = false ;
			else		bdr.append(sep) ;
			
			bdr.append(String.valueOf(t)) ;
		}
		
		return bdr.toString() ;
	}
	
	final static public <T extends Object> String toString(Collection<T> c, String sep)
	{
		return c==null ? null : toString(c.toArray(), sep) ;
	}
	
	final static public String[] asArray(String src, String sep)
	{
		if(src==null) return null ;
		
		StringTokenizer st = new StringTokenizer(src, sep, false) ;
		String[] arr = new String[st.countTokens()] ;
		
		int i=0 ;
		while(st.hasMoreTokens())
			arr[i++] = st.nextToken().trim() ;
		
		return arr ;
	}
	
	final static public List<String> asList(String src, String sep)
	{
		if(src==null) return null ;
		
		StringTokenizer st = new StringTokenizer(src, sep, false) ;
		List<String> list = new ArrayList<String>() ;
		
		while(st.hasMoreTokens())
			list.add(st.nextToken().trim()) ;
		
		return list ;
	}
	
	final static public String[] splitQuote(String line)
	throws Exception
	{
		StringReader sr = new StringReader(line) ;
		StreamTokenizer st = new StreamTokenizer(sr) ;
		st.ordinaryChar('!') ;
		st.wordChars('!', '!') ;
		st.ordinaryChars(35, 255) ;
		st.wordChars(35, 255) ;
		st.quoteChar('"') ;
		
		List<String> list = new ArrayList<String>() ;
		
		int chr = -1 ;
		while(true)
		{
			chr = st.nextToken() ;
			if(chr==StreamTokenizer.TT_EOL || chr==StreamTokenizer.TT_EOF)
				break ;
			
			String token = chr==StreamTokenizer.TT_NUMBER ? String.valueOf(st.nval) : st.sval ;
			if(token!=null) list.add(token) ;
		}
		
		return list.toArray(new String[list.size()]) ;
	}
	
	final static public String insertWhiteSpaceAfter(String src, char... seps)
	{
		for(char sep : seps)
			src = StringUtil.replace(src, sep+"", sep+" ") ;
		
		return src ;	
	}
	
	/**
	 * FOR FUTURE USE
	 */
//	final static public String escape(String src)
//	{
//		//FIXME: is there equivalent in JDOM ??
//		if(src==null) return null ;
//
//		StringBuilder bdr = new StringBuilder() ;
//
//		for(char c : src.toCharArray())
//		{
//			if(c=='&')			bdr.append("&amp;") ;
//			else if(c=='<')		bdr.append("&lt;") ;
//			else if(c=='>')		bdr.append("&gt;") ;
//			else if(c=='\'')	bdr.append("&apos;") ;
//			else if(c=='\"')	bdr.append("&quot;") ;
//			else bdr.append(c) ;
//		}
//
//		return bdr.toString() ;			
//	}
	
//	final static public String doubleCote(String src)
//	{
//		if(src==null) return null ;
//
//		StringBuilder bdr = new StringBuilder() ;
//
//		for(char c : src.toCharArray())
//		{
//			if(c=='\'') bdr.append(c) ;
//			bdr.append(c) ;
//		}
//
//		return bdr.toString() ;
//	}
	
//	final static public String[] toArray(String src, String sep)
//	{
//		if(src==null) return null ;
//		
//		StringTokenizer st = new StringTokenizer(src, sep, false) ;
//		String[] arr = new String[st.countTokens()] ;
//		
//		int i=0 ;
//		while(st.hasMoreTokens())
//			arr[i++] = st.nextToken() ;
//		
//		return arr ;
//	}
}