/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.command;

import org.azuki.framework.command.ICommand.Priority;
import org.azuki.framework.object.Locator;

/**
 * This class is an implementation of the Command Design Pattern.
 * This implmentation adds support for :
 * 		- Asynchronous processing
 * 		- Callback
 *      - Queuing 
 *      
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class Command
{
	static private Impl imp = null;
	final static private Impl imp()
	{ 
		if(imp!=null) return imp ;
		try
		{
			imp = (Impl)Locator.create(Locator.OBJECT_COMMAND) ;
		}
		catch(Exception e)
		{
			e.printStackTrace() ; 
		}
		return imp ;
	}
	
	
	final static public IQueue mainQueue()
	{
		return imp().mainQueue() ;
	}

	final static public IQueue getQueue(String qname)
	{
		return imp().getQueue(qname) ;
	}
	
	/**
	 * Direct call to the <code>process</code> method of the <code>ICommand</code> object.
	 * Invocation is synchronous and performed by the same thread using the same thread context.
	 * 
	 * @param  icmd The command pattern object containing the method to process.
	 * @return The <code>ICommand</code>'s <code>process</code> method return. 
	 */		
	final static public Object call(ICommand icmd)
	throws Exception
	{
		// Direct call
		return icmd.process() ;
	}
	
	/**
	 * Deferred synchronous call to the <code>process</code> method of the command object.
	 * Invocation is synchronous but performed by another thread using a different thread context.
	 * 
	 * @param  icmd The command pattern object containing the method to process.
	 * @return The <code>ICommand</code>'s <code>process</code> method return. 
	 */
	final static public Object invoke(ICommand icmd)
	throws Exception
	{
		return imp().invoke(icmd) ;
	}

	/**
	 * Deferred synchronous call to the <code>process</code> method of the command object.
	 * Invocation is synchronous but performed by another thread using a different thread context.
	 * The caller has specified a queue name and a priority for the command object processing.
	 * 
	 * @param  icmd  The command pattern object containing the method to process.
	 * @param  queue The queue object
	 * @param  pty   The priority 
	 * @return The <code>ICommand</code>'s <code>process</code> method return. 
	 */
	final static public Object invokeQueued(ICommand icmd, IQueue queue, Priority pty)
	throws Exception
	{
		return imp().invokeQueued(icmd, queue, pty) ;
	}

	/**
	 * Asynchronous call to the <code>process</code> method of the <code>ICommand</code> object.
	 * Invocation is asynchronous and performed by another thread using a different thread context.
	 * 
	 * @param  icmd The command pattern object containing the method to process.
	 */
	final static public void send(ICommand icmd)
	{
		imp().send(icmd) ;
	}

	/**
	 * Asynchronous call to the <code>process</code> method of the command object.
	 * Invocation is asynchronous and performed by another thread using a different thread context.
	 * The caller has specified a queue name and a priority for the command object processing.
	 * 
	 * @param  icmd  The command pattern object containing the method to process.
	 * @param  queue The queue object
	 * @param  pty   The priority 
	 */
	final static public void sendQueued(ICommand icmd, IQueue queue, Priority pty)
	{
		imp().sendQueued(icmd, queue, pty) ;
	}

	
	/**
	 * Asynchronous call to the <code>process</code> method of the command object.
	 * Invocation is asynchronous and performed by another thread using a different thread context.
	 * The caller has specified a <code>ICallback</code> object that will be called by the framework
	 * at the completion of the <code>ICommand</code>'s <code>process</code> method execution.
	 * 
	 * @param  icmd   The command pattern object containing the method to process.
	 * @param  cback  The object that will be call by the framework at the completion of
	 *                the <code>ICommand</code>'s <code>process</code> method execution.
	 */
	final static public void sendCallback(ICommand icmd, ICallback cback)
	{
		imp().sendCallback(icmd, cback) ;
	}

	/**
	 * Asynchronous call to the <code>process</code> method of the command object.
	 * Invocation is asynchronous and performed by another thread using a different thread context.
	 * The caller has specified a queue name and a priority for the command object processing.
	 * The caller has also specified a <code>ICallback</code> object that will be called by the framework
	 * at the completion of the <code>ICommand</code>'s <code>process</code> method execution.
	 * 
	 * @param  icmd   The command pattern object containing the method to process.
	 * @param  queue  The queue object 
	 * @param  pty    The priority 
	 * @param  cback  The object that will be call by the framework at the completion of
	 *                the <code>ICommand</code>'s <code>process</code> method execution.
	 */
	final static public void sendQueuedCallback(ICommand icmd, IQueue queue, Priority pty, ICallback cback)
	{
		imp().sendQueuedCallback(icmd, queue, pty, cback) ;
	}
	
	public interface Impl
	{
		public IQueue mainQueue() ;
		
		public IQueue getQueue(String name) ;
				
		public Object invoke(ICommand icmd)
		throws Exception ;
	
		public Object invokeQueued(ICommand icmd, IQueue queue, Priority pty)
		throws Exception ;
	
		public void send(ICommand icmd) ;
	
		public void sendQueued(ICommand icmd, IQueue queue, Priority pty) ;
	
		public void sendCallback(ICommand icmd, ICallback cback) ;
	
		public void sendQueuedCallback(ICommand icmd, IQueue queue, Priority pty, ICallback cback) ;
	}	
}